from .logging import Logging
from flask import request
from flask.wrappers import Response
import functools
import json
from uuid import UUID
import datetime

def uuid_to_str(object):
    '''
    Converts and uuid object to string
    '''
    if isinstance(object, UUID):
        return str(object)

def log_request(logger):
    """
        Decorates a flask route to log request and response
    """

    def decorator(func):

        @functools.wraps(func)
        def wrapper_log_request(*args, **kwargs):
            request_data = request.get_json(silent=True)

            logger.log_request(request.url, request.method, request_data)

            response_data = func(*args, **kwargs)
            
            if type(response_data) is Response:
                response_data_encoded = response_data.response[0].decode()
                response_data_encoded = json.dumps(response_data_encoded,
                    default=uuid_to_str)
            if type(response_data) is dict:
                response_data_encoded = json.dumps(response_data,
                    default=uuid_to_str)

            logger.log_request(
                request.url,
                request.method,
                response_data_encoded,
                request_type='RESPONSE'
            )

            return response_data

        return wrapper_log_request

    return decorator
